//
//  BasePanoViewController.swift
//  PhoenixDemo
//
//  Created by zhangpeng26 on 2021/4/1.
//

import Phoenix
import UIKit

class BasePanoImageViewController: UIViewController {
    private var configSegment: UISegmentedControl?
    private var viewSegment: UISegmentedControl?
    
    var panoView: PanoView?
    var segmentConfigs: [String] = []
    var isDebug = false
    var enableCache = true
    var tileUpdateDuration: Double = 0.5
    private var switchItems: [[String]] = [["pinch", "motion", "pitch", "pan"]]
    private var switchStacks: [UIStackView] = []
    private var buttonItem: [String] = ["shot", "rotate", "auto"]
    private let buttonStack = UIStackView()
    private let snapShotImage = UIImageView()
    
    override func viewDidLoad() {
        super.viewDidLoad()

        setupConfig()
        setupPanoView()
        setupConfigSegment()
        setupViewSegment()
        setupHotspot()
        setupSwitch()
        setupButton()
        
        view.addSubview(snapShotImage)
    }

    func setupConfig() {
    }

    private func setupPanoView() {
        let panoConfig = PanoConfiguration(resource: segmentConfigs.first!)
        
        if let config = panoConfig {
            config.enableEnterAnimation = false
            config.sceneBackgroundColor = .black
            config.tileUpdateDuration = tileUpdateDuration
            panoView = PanoView(frame: view.frame,
                                configuration: config)
            panoView?.panoDelegate = self
            view.addSubview(panoView!)
        }
    }

    private func setupConfigSegment() {
        var segItem: [String] = []
        for index in 1 ... segmentConfigs.count {
            segItem.append("配置\(index)")
        }

        configSegment = UISegmentedControl(items: segItem)
        configSegment?.selectedSegmentIndex = 0

        configSegment?.addTarget(self,
                                 action: #selector(configSegmentedValueChanged(_:)),
                                 for: .valueChanged)

        view?.addSubview(configSegment!)
    }

    private func setupViewSegment() {
        var segItem: [String] = []
        for index in 1 ... segmentConfigs.count {
            segItem.append("场景\(index)")
        }
        
        viewSegment = UISegmentedControl(items: segItem)
        viewSegment?.selectedSegmentIndex = 0
        
        viewSegment?.addTarget(self,
                                 action: #selector(viewSegmentedValueChanged(_:)),
                                 for: .valueChanged)
        
        view?.addSubview(viewSegment!)
    }
    
    private func setupSwitch() {
        for rowIndex in 0 ..< switchItems.count {
            let stack = UIStackView()
            stack.axis = .horizontal

            for index in 0 ..< switchItems[rowIndex].count {
                let switchUI = UISwitch()

                if rowIndex == 0 && index == 1 {
                    switchUI.isOn = false
                } else {
                    switchUI.isOn = true
                }
                
                switchUI.tag = rowIndex * 100 + index
                switchUI.addTarget(self,
                                   action: #selector(switchValueChanged(_:)),
                                   for: .valueChanged)
                stack.addArrangedSubview(switchUI)

                let lable = UILabel()
                lable.text = switchItems[rowIndex][index]
                stack.addArrangedSubview(lable)
            }

            view?.addSubview(stack)
            switchStacks.append(stack)
        }
    }
    private func setupButton() {
        buttonStack.axis = .horizontal
        buttonStack.spacing = 10
        for index in 0 ..< buttonItem.count {
            let button = UIButton()
            button.backgroundColor = .systemBlue
            button.setTitle(buttonItem[index], for: .normal)
            button.setTitleColor(.white, for: .normal)
            button.addTarget(self,
                             action: #selector(buttonClicked(_:)),
                             for: .touchUpInside)
            buttonStack.addArrangedSubview(button)
        }
        
        view?.addSubview(buttonStack)
    }
    
    @objc private func buttonClicked(_ sender: UIButton!) {
        print("buttonClicked: \(String(describing: sender.title(for: .normal)))")
        
        if sender.title(for: .normal) == "shot" {
        } else if sender.title(for: .normal) == "rotate" {
            let animation = CABasicAnimation(keyPath: "eulerAngles.y")
            animation.byValue = degreeToRadians(angle: 360)
            animation.duration = 10

            panoView?.camera?.add(animation, forKey: "camera")
        } else if sender.title(for: .normal) == "auto" {
            panoView?.startAutoRotate(duration: 100)
        }
    }
    
    @objc private func switchValueChanged(_ sender: UISwitch!) {
        print("switchValueChanged: \(String(describing: sender.tag)), \(sender.isOn ? "on" : "off")")

        let rowIndex = sender.tag / 100
        let columnIndex = sender.tag % 10

        if rowIndex == 0 {
            if columnIndex == 0 {
                panoView?.isPinchEnabled = sender.isOn
            } else if columnIndex == 1 {
                panoView?.isMotionEnabled = sender.isOn
            } else if columnIndex == 2 {
                panoView?.isPitchEnabled = sender.isOn
            } else if columnIndex == 3 {
                panoView?.isPanEnabled = sender.isOn
            }
        }
    }

    private func setupHotspot() {
        for index in 0...1 {
            let helloHotspot = HelloPanoHotspot()
            helloHotspot.frame = CGRect(x: 0, y: 0, width: 42, height: 42)
            let hotspot = PanoViewHotspot(view: helloHotspot, x: CGFloat(index * 10), y: -20, z: -80)
            panoView?.add(hotspot: hotspot)
        }
    }

    @objc private func configSegmentedValueChanged(_ sender: UISegmentedControl!) {
        print("Selected Segment Index is : \(sender.selectedSegmentIndex)")

        let panoConfig = PanoConfiguration(resource: segmentConfigs[sender.selectedSegmentIndex])

        if let config = panoConfig {
            panoView?.updateConfiguration(configuration: config)
        }
    }
    
    @objc private func viewSegmentedValueChanged(_ sender: UISegmentedControl!) {
        print("Selected Segment Index is : \(sender.selectedSegmentIndex)")
        
        let panoConfig = PanoConfiguration(resource: segmentConfigs[sender.selectedSegmentIndex])
        
        if let config = panoConfig {
            panoView?.switchView(configuration: config)
        }
    }

    override func viewDidLayoutSubviews() {
        var topOffset = view.safeAreaInsets.top

        panoView?.frame = CGRect(x: 0,
                                 y: view.safeAreaInsets.top,
                                 width: view.frame.width,
                                 height: view.frame.height - view.safeAreaInsets.top)

        configSegment?.frame = CGRect(x: 0,
                                      y: topOffset,
                                      width: CGFloat(50 * segmentConfigs.count),
                                      height: 30)

        topOffset += 40
        
        viewSegment?.frame = CGRect(x: 0,
                                      y: topOffset,
                                      width: CGFloat(50 * segmentConfigs.count),
                                      height: 30)
        
        for stack in switchStacks {
            topOffset += 40
            stack.frame = CGRect(x: 0,
                                 y: Int(topOffset),
                                 width: Int(view.bounds.width),
                                 height: 50)
        }
        
        topOffset += 50
        buttonStack.frame = CGRect(x: 0,
                                   y: Int(topOffset),
                                   width: 60 * buttonItem.count,
                                   height: 50)
        
        topOffset += 50
        snapShotImage.frame = CGRect(x: 0,
                                     y: topOffset,
                                     width: view.bounds.width / 5,
                                     height: view.bounds.height / 5)
    }
    
    func degreeToRadians(angle: Float) -> Float {
        return angle / 180.0 * Float.pi
    }
}

extension BasePanoImageViewController: PanoViewDelegate {
    func panoViewLoadStart(_ view: PanoView) {
        print("[pano][panoViewLoadStart]pano view load start")
    }
    
    func panoViewLoadFinish(_ view: PanoView, error: Error?) {
        print("[pano][panoViewLoadFinish]pano view load finish,error:\(String(describing: error))")
    }
    
    func panoViewLoadTileFailed(_ view: PanoView, error: Error?) {
        print("[pano][panoViewLoadTileFailed]pano tile load finish,error:\(String(describing: error))")
    }
    
    func panoViewPanGestureBegin(_ view: PanoView) {
        panoView?.stopAutoRotate()
    }
    
    func panoViewPanGestureEnd(_ view: PanoView) {
    }
}
