//
//  PanoVideoSphereViewController.swift
//  PhoenixDemo
//
//  Created by zhangpeng26 on 2021/4/1.
//

import AVFoundation
import Phoenix
import UIKit

class PanoVideoSphereViewController: UIViewController {
    private var panoView: PanoView?
    private var player: AVPlayer!
    private var avPLayerItem: AVPlayerItem!
    private var segment: UISegmentedControl?

    var isDebug = false

    override func viewDidLoad() {
        super.viewDidLoad()

        setupPanoView()

        setupConfigSegment()
    }

    private func setupPanoView() {
        let urlString = "测试视频.mp4".addingPercentEncoding(withAllowedCharacters: .urlPathAllowed)

        guard let url = URL(string: "https://repos3d-dev.cdn.bcebos.com/dev/media/pano/" + urlString!) else {
            return
        }

        avPLayerItem = AVPlayerItem(asset: AVAsset(url: url))
        player = AVPlayer(playerItem: avPLayerItem)

        player.play()

        let vrResource = VRResource()
        vrResource.setResourceType(type: .panoVideo)
        vrResource.setRenderType(type: .sphere)
        vrResource.setResource(resource: avPLayerItem!)

        let config = PanoConfiguration(vrResource: vrResource)
        
        panoView = PanoView(frame: view.frame,
                            configuration: config)

        view.addSubview(panoView!)
    }
    
    private func setupConfigSegment() {
        segment = UISegmentedControl(items: ["play", "pause"])
        segment?.selectedSegmentIndex = 0
        view.addSubview(segment!)

        segment?.addTarget(self,
                           action: #selector(segmentedValueChanged(_:)),
                           for: .valueChanged)
    }

    @objc private func segmentedValueChanged(_ sender: UISegmentedControl!) {
        print("Selected Segment Index is : \(sender.selectedSegmentIndex)")

        if sender.selectedSegmentIndex == 0 {
            player.play()
        } else {
            player.pause()
        }
    }

    override func viewDidLayoutSubviews() {
        panoView?.frame = CGRect(x: view.frame.minX,
                                 y: view.safeAreaInsets.top,
                                 width: view.frame.width,
                                 height: view.frame.height - view.safeAreaInsets.top)

        segment?.frame = CGRect(x: view.frame.minX,
                                y: view.safeAreaInsets.top,
                                width: CGFloat(100),
                                height: 30)
    }
}
