//
//  BaseSpinViewController.swift
//  PhoenixDemo
//
//  Created by zhangpeng26 on 2021/4/27.
//

import Foundation

import Phoenix
import UIKit

class BaseSpinViewController: UIViewController {
    private var configSegment: UISegmentedControl?

    var spinView: SpinView?
    var segmentConfigs: [String] = []
    var isDebug = false
    var enableCache = true
    var tileUpdateDuration: Double = 500
    private var switchItems: [[String]] = [["zoom", "pan", "reversalX", "reversalY"],
                                           ["spin"]]
    private var switchStacks: [UIStackView] = []
    private var buttonItems: [String] = ["reset", "rotate", "round", "stop", "center", "pan"]
    private let buttonStack = UIStackView()
    private var clolumnCount = 0
    private var startColumn = 0
    private var enableEnterAnimation = true

    override func viewDidLoad() {
        super.viewDidLoad()

        setupConfig()
        setupSpinView()
        setupConfigSegment()
        setupSwitch()
        setupButton()
    }

    func setupConfig() {
    }

    private func enterAnimation() {
        guard let columnCount = spinView?.columnCount else {
            return
        }

        spinView?.zoom(to: 0)

        let duration: Double = 2
        var animations: [CAAnimation] = []

        if startColumn == 0 {
            let columnAnimation = CABasicAnimation(keyPath: SpinAnimationProperty.column)
            columnAnimation.fromValue = 0
            columnAnimation.toValue = columnCount
            animations.append(columnAnimation)
        } else {
            let duration1 = Double(startColumn) / Double(columnCount) * Double(duration)

            let columnAnimation1 = CABasicAnimation(keyPath: SpinAnimationProperty.column)
            columnAnimation1.fromValue = startColumn
            columnAnimation1.toValue = columnCount
            columnAnimation1.beginTime = 0
            columnAnimation1.duration = duration - duration1
            animations.append(columnAnimation1)

            let columnAnimation2 = CABasicAnimation(keyPath: SpinAnimationProperty.column)
            columnAnimation2.fromValue = 0
            columnAnimation2.toValue = startColumn
            columnAnimation2.beginTime = duration - duration1
            columnAnimation2.duration = duration1
            animations.append(columnAnimation2)
        }

        let scaleAnimation = CABasicAnimation(keyPath: SpinAnimationProperty.scale)
        scaleAnimation.fromValue = 0
        scaleAnimation.toValue = 1
        animations.append(scaleAnimation)

        let group = CAAnimationGroup()
        group.animations = animations
        group.duration = duration
        group.delegate = self
        group.fillMode = .forwards
        spinView?.add(group, forKey: "group")
    }

    private func setupSpinView() {
        let spinConfig = SpinConfiguration(resource: self.segmentConfigs.first!)
        
        if let config = spinConfig {
            config.initializedRow = 0
            config.initializedColumn = self.startColumn
            config.tileUpdateDuration = self.tileUpdateDuration
            
            spinView = SpinView(frame: view.frame)
            spinView?.updateConfiguration(configuration: config)
            
            self.setupHotspot()
            self.spinView?.delegate = self
            
            if self.enableEnterAnimation {
                self.spinView?.hideHotspot()
            }
            
            view.addSubview(spinView!)
        }
    }

    private func setupHotspot() {
        let helloHotspot = HelloPanoHotspot()
        helloHotspot.frame = CGRect(x: 0, y: 0, width: 50, height: 50)

        let hotspot = SpinHotspot(view: helloHotspot, row: 0, column: 0, x: 0.5, y: 0.5)
        spinView?.add(hotspot: hotspot)
    }

    private func setupConfigSegment() {
        var segItem: [String] = []
        for index in 1 ... segmentConfigs.count {
            segItem.append("配置\(index)")
        }

        configSegment = UISegmentedControl(items: segItem)
        configSegment?.selectedSegmentIndex = 0

        configSegment?.addTarget(self,
                                 action: #selector(segmentedValueChanged(_:)),
                                 for: .valueChanged)

        view?.addSubview(configSegment!)
    }

    private func setupSwitch() {
        for rowIndex in 0 ..< switchItems.count {
            let stack = UIStackView()
            stack.axis = .horizontal

            for index in 0 ..< switchItems[rowIndex].count {
                let switchUI = UISwitch()

                if rowIndex == 0 && (index == 2 || index == 3) {
                    switchUI.isOn = false
                } else {
                    switchUI.isOn = true
                }

                switchUI.tag = rowIndex * 100 + index
                switchUI.addTarget(self,
                                   action: #selector(switchValueChanged(_:)),
                                   for: .valueChanged)
                stack.addArrangedSubview(switchUI)

                let lable = UILabel()
                lable.text = switchItems[rowIndex][index]
                stack.addArrangedSubview(lable)
            }

            view?.addSubview(stack)
            switchStacks.append(stack)
        }
    }

    private func setupButton() {
        buttonStack.axis = .horizontal
        buttonStack.spacing = 10
        for index in 0 ..< buttonItems.count {
            let button = UIButton()
            button.backgroundColor = .systemBlue
            button.setTitle(buttonItems[index], for: .normal)
            button.setTitleColor(.white, for: .normal)
            button.addTarget(self,
                             action: #selector(buttonClicked(_:)),
                             for: .touchUpInside)
            buttonStack.addArrangedSubview(button)
        }

        view?.addSubview(buttonStack)
    }

    @objc private func segmentedValueChanged(_ sender: UISegmentedControl!) {
        print("Selected Segment Index is : \(sender.selectedSegmentIndex)")

        let spinConfig = SpinConfiguration(resource: segmentConfigs[sender.selectedSegmentIndex])

        if let config = spinConfig {
            spinView?.updateConfiguration(configuration: config)
        }
    }

    @objc private func switchValueChanged(_ sender: UISwitch!) {
        print("switchValueChanged: \(String(describing: sender.tag)), \(sender.isOn ? "on" : "off")")

        let rowIndex = sender.tag / 100
        let columnIndex = sender.tag % 10

        if rowIndex == 0 {
            if columnIndex == 0 {
                spinView?.isZoomEnabled = sender.isOn
            } else if columnIndex == 1 {
                spinView?.isPanEnabled = sender.isOn
            } else if columnIndex == 2 {
                spinView?.isVerticalReversal = sender.isOn
            } else if columnIndex == 3 {
                spinView?.isHorizontalReversal = sender.isOn
            }
        } else if rowIndex == 1 {
            if columnIndex == 0 {
                spinView?.isSpinEnabled = sender.isOn
            }
        }
    }

    @objc private func buttonClicked(_ sender: UIButton!) {
        print("buttonClicked: \(String(describing: sender.title(for: .normal)))")

        if sender.title(for: .normal) == "reset" {
            spinView?.reset()
        } else if sender.title(for: .normal) == "rotate" {
        } else if sender.title(for: .normal) == "round" {
            spinView?.spinRound()
        } else if sender.title(for: .normal) == "stop" {
            spinView?.removeAnimation(forKey: "group")
        } else if sender.title(for: .normal) == "center" {
            spinView?.panToCenter()
        } else if sender.title(for: .normal) == "pan" {
            spinView?.pan(to: CGPoint(x: 0.5, y: 0.5))
        }
    }

    override func viewDidLayoutSubviews() {
        var topOffset = view.safeAreaInsets.top
        spinView?.frame = CGRect(x: 0,
                                 y: view.safeAreaInsets.top,
                                 width: view.frame.width,
                                 height: view.frame.height - view.safeAreaInsets.top)

        configSegment?.frame = CGRect(x: 0,
                                      y: topOffset,
                                      width: CGFloat(50 * segmentConfigs.count),
                                      height: 30)

        for stack in switchStacks {
            topOffset += 40
            stack.frame = CGRect(x: 0,
                                 y: Int(topOffset),
                                 width: Int(view.bounds.width),
                                 height: 50)
        }

        topOffset += 50
        buttonStack.frame = CGRect(x: 0,
                                   y: Int(topOffset),
                                   width: 60 * buttonItems.count,
                                   height: 50)
    }
}

extension BaseSpinViewController: CAAnimationDelegate {
    public func animationDidStop(_ anim: CAAnimation, finished flag: Bool) {
        spinView?.zoom(to: 1)
        spinView?.spinTo(row: nil, column: startColumn)
        if enableEnterAnimation {
            spinView?.showHotspot()
        }
    }
}

extension BaseSpinViewController: SpinViewDelegate {
    func spinViewLoadCoverFinish(_ view: SpinView, error: Error?) {
        print("[spin][spinViewLoadCoverFinish]spin cover load finish,error:\(String(describing: error))")
    }
    
    func spinViewScaleChanged(_ view: SpinView, scale: CGFloat) {
    }

    func spinViewRoundStart(_ view: SpinView) {
    }

    func spinViewRoundFinish(_ view: SpinView) {
    }

    func spinViewPanGestureBegin(_ view: SpinView) {
        spinView?.removeAnimation(forKey: "group")
    }

    func spinViewPanGestureEnd(_ view: SpinView) {
    }

    func spinViewLoadStart(_ view: SpinView) {
        print("[spin][spinViewLoadStart]spin view load start")
    }

    func spinViewLoadFinish(_ view: SpinView, error: Error?) {
        print("[spin][spinViewLoadFinish]spin view load finish,error:\(String(describing: error))")

        if error == nil {
            if enableEnterAnimation {
                enterAnimation()
            }
        }
    }

    func spinViewLoadTileFailed(_ view: SpinView, error: Error?) {
        print("[spin][spinViewLoadTileFailed]spin tile load finish,error:\(String(describing: error))")
    }
}
