//
//  PanoTestViewController.swift
//  PhoenixDemo
//
//  Created by zhangpeng26 on 2021/5/29.
//

import UIKit
import SceneKit
import Phoenix

class PanoTestViewController: UIViewController {
    var panoView: PanoView?
    
    var rotateTimer: Timer?
    
    var zoomInTimer: Timer?
    var zoomInTestTimer: Timer?
    var zoomInCount = 0
    
    var dragTimer: Timer?
    var dragTestTimer: Timer?
    var dragCount = 0
    
    var zoomOutTimer: Timer?
    var zoomOutTestTimer: Timer?
    var zoomOutCount = 0
    
    private let config: String = """
    {
        "resource_type": "pano",
        "render_type": "cube-multi",
        "resource": {
            "file": "panos/input.tiles/%s/l%l/%v/l%l_%s_%v_%h.jpg",
            "url": "https://bj.bcebos.com/v1/repos3d/penglai/pano_packages/pack_pano_default_07/pieces/panos/input.tiles/",
            "tile_size": [
                512,
                1152,
                2304,
                4608
            ]
        }
    }
    """
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        setupPanoView()
        
        rotateTimer = Timer.scheduledTimer(withTimeInterval: 20, repeats: false, block: {[weak self] _ in
            self?.startRotate()
        })
        
        zoomInTimer = Timer.scheduledTimer(withTimeInterval: 50, repeats: false, block: {[weak self] _ in
            self?.startZoomInTest()
        })
        
        dragTimer = Timer.scheduledTimer(withTimeInterval: 70, repeats: false, block: {[weak self] _ in
            self?.startDragTest()
        })
        
        zoomOutTimer = Timer.scheduledTimer(withTimeInterval: 90, repeats: false, block: {[weak self] _ in
            self?.startZoomOutTest()
        })
    }
    
    private func setupPanoView() {
        let panoConfig = PanoConfiguration(resource: config)
        
        if let config = panoConfig {
            panoView = PanoView(frame: view.frame,
                                configuration: config)
            view.addSubview(panoView!)
        }
    }
    
    override func viewDidLayoutSubviews() {
        panoView?.frame = CGRect(x: 0,
                                 y: view.safeAreaInsets.top,
                                 width: view.frame.width,
                                 height: view.frame.height - view.safeAreaInsets.top)
    }
    
    
    func degreeToRadians(angle: Float) -> Float {
        return angle / 180.0 * Float.pi
    }
    
    func startRotate() {
        let animation = CABasicAnimation(keyPath: "eulerAngles.y")
        animation.fromValue = 0
        animation.toValue = degreeToRadians(angle: 360)
        animation.duration = 10
        animation.repeatCount = 2
        
        panoView?.camera?.add(animation, forKey: nil)
    }
    
    func startZoomInTest() {
        zoomInTestTimer = Timer.scheduledTimer(withTimeInterval: 2, repeats: true, block: {[weak self] _ in
            self?.zoomIn()
        })
    }
    
    func zoomIn() {
        panoView?.camera?.fov -= 5;
        zoomInCount += 1
        
        if zoomInCount == 5 {
            zoomInTestTimer?.invalidate()
        }
    }
    
    func startDragTest() {
        dragTestTimer = Timer.scheduledTimer(withTimeInterval: 2, repeats: true, block: {[weak self] _ in
            self?.drag()
        })
    }
    
    func drag() {
        panoView?.camera?.eulerAngles.y += degreeToRadians(angle: 10)
        dragCount += 1
        
        if dragCount == 5 {
            dragTestTimer?.invalidate()
        }
    }
    
    func startZoomOutTest() {
        zoomOutTestTimer = Timer.scheduledTimer(withTimeInterval: 2, repeats: true, block: {[weak self] _ in
            self?.zoomOut()
        })
    }
    
    func zoomOut() {
        panoView?.camera?.fov += 5;
        zoomOutCount += 1
        
        if zoomOutCount == 5 {
            zoomOutTestTimer?.invalidate()
        }
    }
}

