//
//  SpinTestViewController.swift
//  PhoenixDemo
//
//  Created by zhangpeng26 on 2021/5/29.
//

import UIKit
import Phoenix

class SpinTestViewController: UIViewController {
    var spinView: SpinView?
    
    var rotateTimer: Timer?
    
    var zoomInTimer: Timer?
    var zoomInTestTimer: Timer?
    var zoomInCount = 0
    
    var dragTimer: Timer?
    var dragTestTimer: Timer?
    var dragCount = 0
    
    var zoomOutTimer: Timer?
    var zoomOutTestTimer: Timer?
    var zoomOutCount = 0
    
    var drag2Timer: Timer?
    var drag2TestTimer: Timer?
    var drag2Count = 0
    
    private let config: String = """
    {
        "resource_type": "spin",
        "render_type": "multi-tile",
        "resource": {
            "tile_path": "https://repos3d.cdn.bcebos.com/penglai/materials/ma_90f839f86/pieces/origin/spin_720_scene2",
            "cover_image": "${row}_${col}_cover.jpg",
            "extension": "jpg",
            "tile_file": "${row}_${col}_files",
            "tile_size": 510,
            "width": 3000,
            "height": 2000,
            "col_count": 36,
            "row_count": 2,
            "min_level": 10,
            "cover_width": 512,
            "cover_height": 314
        }
    }
    """
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        setupSpinView()
        
        rotateTimer = Timer.scheduledTimer(withTimeInterval: 20, repeats: false, block: {[weak self] _ in
            self?.startRotate()
        })
        
        zoomInTimer = Timer.scheduledTimer(withTimeInterval: 50, repeats: false, block: {[weak self] _ in
            self?.startZoomInTest()
        })
        
        dragTimer = Timer.scheduledTimer(withTimeInterval: 70, repeats: false, block: {[weak self] _ in
            self?.startDragTest()
        })
        
        zoomOutTimer = Timer.scheduledTimer(withTimeInterval: 90, repeats: false, block: {[weak self] _ in
            self?.startZoomOutTest()
        })
        
        drag2Timer = Timer.scheduledTimer(withTimeInterval: 110, repeats: false, block: {[weak self] _ in
            self?.startDrag2Test()
        })
    }
    
    private func setupSpinView() {
        let spinConfig = SpinConfiguration(resource: config)
        
        if let config = spinConfig {
            config.initializedColumn = 0
            spinView = SpinView(frame: view.frame, configuration: config)
            view.addSubview(spinView!)
        }
    }
    
    override func viewDidLayoutSubviews() {
        spinView?.frame = CGRect(x: 0,
                                 y: view.safeAreaInsets.top,
                                 width: view.frame.width,
                                 height: view.frame.height - view.safeAreaInsets.top)
    }
    
    func startRotate() {
        let rotateAnimation = CABasicAnimation(keyPath: SpinAnimationProperty.column)
        rotateAnimation.fromValue = 0
        rotateAnimation.toValue = spinView?.columnCount
        rotateAnimation.duration = 10
        rotateAnimation.repeatCount = 2
        
        spinView?.add(rotateAnimation, forKey: nil)
    }
    
    func startZoomInTest() {
        zoomInTestTimer = Timer.scheduledTimer(withTimeInterval: 2, repeats: true, block: {[weak self] _ in
            self?.zoomIn()
        })
    }
    
    func zoomIn() {
        spinView?.zoom(by: 1.2)
        
        zoomInCount += 1
        
        if zoomInCount == 5 {
            zoomInTestTimer?.invalidate()
        }
    }
    
    func startDragTest() {
        dragTestTimer = Timer.scheduledTimer(withTimeInterval: 2, repeats: true, block: {[weak self] _ in
            self?.drag()
        })
    }
    
    func drag() {
        spinView?.pan(by: CGPoint(x: 5, y: 0))
        
        dragCount += 1
        
        if dragCount == 5 {
            dragTestTimer?.invalidate()
        }
    }
    
    func startZoomOutTest() {
        zoomOutTestTimer = Timer.scheduledTimer(withTimeInterval: 2, repeats: true, block: {[weak self] _ in
            self?.zoomOut()
        })
    }
    
    func zoomOut() {
        spinView?.zoom(by: 0.8)
        zoomOutCount += 1
        
        if zoomOutCount == 5 {
            zoomOutTestTimer?.invalidate()
        }
    }
    
    func startDrag2Test() {
        drag2TestTimer = Timer.scheduledTimer(withTimeInterval: 2, repeats: true, block: {[weak self] _ in
            self?.drag2()
        })
    }
    
    func drag2() {
        spinView?.spinBy(row: 0, column: 3)
        
        drag2Count += 1
        
        if drag2Count == 5 {
            drag2TestTimer?.invalidate()
        }
    }
}
